#ifndef POWERSAVE_POWERLIB_H
#define POWERSAVE_POWERLIB_H

/** @defgroup libpower Libpower
 *
*/


#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <syslog.h>
#include <stdint.h>

/** @defgroup libpower_information Functions to gather acpi/apm information
 * @ingroup libpower
 * this is conform to the ACPI specification 2.0b (2002)
 *
 * <b>IMPORTANT !!!</b>
 * Test all data for UNKNOWN<br>
 * You never know if the kernel or the hw supports specific features!
 * @{ 
 */

#define MAX_FILE_PATH 512 /* XXX: HOpe this is only under /proc/acpi where this is correct. */
#define MAX_LINE_SIZE 1024

#define MAX_BATTERIES 4
	/** Max string length in ACPI Source Language: 200 bytes */
#define MAX_BAT_CHAR_LEN 200

#define MAX_THERMAL_ACTIVE 10

#define NO_ACPI_ERROR -20
#define NO_MODULE_ERROR -10
#define NO_DEVICE_ERROR -5

#define MAX_SUPPORTED_CPUS 32

/** be careful to test every variable on unknown */
#define UNKNOWN -1

	enum BATTERY_PRESENT{PRESENT_YES = 0, PRESENT_NO};
	enum CAPACITY_UNIT{WATT_H = 0, AMP};
	enum BATTERY_TECH{TECH_NON_RECHARGEABLE = 0, TECH_RECHARGEABLE};
	
	/** because of kernel implementations separated in ACPI spec this
	 * is combined to battery_state 
	 * be careful this is kernel definition of batteries states !
	 */
	enum CAPACITY_STATE{STATE_OK, STATE_CRIT, STATE_LOW, STATE_WARN};
#define CHARG_STATE_UNKNOWN	0
#define CHARG_STATE_CHARGING	1
#define CHARG_STATE_DISCHARGING	2
#define CHARG_STATE_CHARG_DISCHARG (CHARG_STATE_CHARGING | CHARG_STATE_DISCHARGING)
	enum AC_ADAPTER_STATE{AC_UNKNOWN, AC_ONLINE, AC_OFFLINE};
	enum DISK_MODE{DISK_FULL_ON, DISK_STANDBY, DISK_SLEEP};
	enum LID_STATE{LID_OPEN, LID_CLOSED};

	/** use for checkACPI function return value! */
	enum ISACPI{NOT_SUPPORTED=-1, APM, ACPI};

	/** modes used for active/passive cooling mode
	 and for the devices' temperature state */
	enum THERM_MODE{OK, ACTIVE, PASSIVE, HOT, CRITICAL};

	enum _COOLING_POLICY{COOLING_MODE_ACTIVE=0, COOLING_MODE_PASSIVE};

/** General battery information */
typedef struct BatteryGeneral{
	int remPerc;
	int remMins;
        /** charging, discharging, charging/discharging, unknown*/
	int charging_state; 
	int rem_charging_mins;
	/*	int capacity_state;*/
	/* ToDo: what about default battery warning low -> I think it is not needed here?!?*/
} BatteryGeneral;


	/** @brief Detailed battery information
	 * only available for ACPI through the getBatteryDetailedInfo function */
typedef struct BatteryDetailed{
	int present;

	/* this is from info (ACPI)*/
	int design_capacity;
	int last_full_capacity;
	int battery_technology;
	int design_voltage;
	int design_capacity_warning;
	int design_capacity_low;
	int capacity_granularity_1;
	int capacity_granularity_2;
	char model_number[MAX_BAT_CHAR_LEN+1];
	char serial_number[MAX_BAT_CHAR_LEN+1];
	char battery_type[MAX_BAT_CHAR_LEN+1];
	char OEM_info[MAX_BAT_CHAR_LEN+1];

	/* this is from status (ACPI)*/
	int power_unit;
	int capacity_state;
	int charging_state;
	int present_rate;
	int remaining_capacity;
	int present_voltage;

	/* this is from alarm*/
	int alarm_limit;


} BatteryDetailed;


typedef struct ThermalDev{

	int present;
	int temperature;

	int cooling_mode;
	int polling_frequency;
	int critical;
	int hot;
	int passive;	
	int active[MAX_THERMAL_ACTIVE];

	int state;
	int tc1;
	int tc2;
	int tsp;
}ThermalDev;




	/** @brief fills the given Battery struct with the current
	 *  battery information	                                                
	 * @param bat pointer to a BatteryGeneral struct
	 * @return integer with result of call
	 * @retval 1 on success
	 * @retval UNKNOWN if no batteries will ever be available on the
	 *         machine, or modules have not been loaded
	 *
	 *  fills the given Battery struct with the current battery information
	 *
	 *  supports acpi and apm
	 *                                                
	 *************************************************/
	int getBatteriesInfo(BatteryGeneral *bat);

	/*   O N L Y   F O R   A C P I ! ! !  */
	/*
	 *                                               
	 *  Functions: 
	 *             int getBatteryDetailedInfo(const int no, BatteryDetailed &bd);
	 *             int setBatteryAlarm(const int percent);
	 *
	 * Only ACPI!
	 * These functions can be used to gain special information on batteries on 
	 * a ACPI system only!
	 * They are also used internally to determine the general battery information
	 * above                                              
	 *                                                
	 *************************************************/

	/** @brief use this method to get amount of batteries in the system
	 *
	 * @param no battery number
	 * @param bd pointer to detailed battery info structure
	 * @return: integer with result of call
	 * @retval 0 SUCCESS
	 * @retval < 0 ERROR
	 * @retval -1 general error
	 * @retval -2 if device no does not exist
	 * @retval -3 if struct has not initialized with memory
	 * @retval NO_ACPI_ERROR if it is not an ACPI system 
	 * @retval NO_MODULE_ERROR if the battery module is not loaded 
	 * @retval NO_DEVICE_ERROR if there is a fan directory, but no device 
	 * @retval > 0 END OF BATTERIES REACHED
	 * 
	 * <b>ACPI only!</b>
	 *
	 * iterate until it's != 0 and check results for bd->present == PRESENT_YES
	 */
	int getBatteryDetailedInfo(const int no, BatteryDetailed *bd);

	/** @brief sets the ACPI battery alarm for all batteries
	 *
	 * <b>ACPI only!</b><br>
	 */
	int setBatteryAlarm(const int percent);
	
	/** @brief returns the current state of the lid
	 * (enum LID_STATE{LID_OPEN, LID_CLOSED})
	 */
	int getLidState(void);

	/**************************************************
	 *                                               
	 *
	 * Functions: int getThrottlingInfo(int &states, int &current);
	 *            int setThrottlingState(const int state);
	 *            int setThrottlingPercent(const int percent);
	 *
	 * Only ACPI!
	 * get/set throttling states (0-8)
	 *
	 *************************************************/
	/** @brief get throttling information (ACPI only) */
	int getThrottlingInfo(int *states, int *current);
	/* not needed any more use setThrottlingPercent*/
	/*	int setThrottlingState(const int state);*/

	/** @brief set throttling percent (ACPI only) */
	int setThrottlingPercent(const int percent);

	int getThrottlingInfoCPU(const int cpu, int *states, int *current);

	/** @brief set throttling state on cpu (ACPI only) */
	int setThrottlingStateCPU (const int cpu, const int state);

	int setThrottlingPercentCPU(const int cpu, const int percent);
	/**************************************************
	 *                                               
	 *
	 * Functions: int getThermalZonesNum();
	 *	      int getThermalZoneTemp(const int zone=0);
	 *
	 * Only ACPI!
	 * determines the thermal temperature for each themperautre zone
	 * 
	 *************************************************/

	/** @brief get the number of thermal zones 
	 *
	 * returns greater than zero on success 
	 * less than zero on failure
	 */
	int getThermalZonesNum(void);

	int getThermalZoneTemp(const int zone);

	int getThermalZoneState(int zone);

	/** @brief get a thermal zone 
	 *
	 * @return integer value with result of call 
	 * @retval zero or greater on success 
	 * @retval 1 if no ACPI 
	 * @retval 2 if no thermal directory (module probably not loaded) 
	 * @retval 3 if thermal device no. num does not exist 
	 * @retval 4 if td is not allocated 
	 */
	int getThermalZone(const int zone, ThermalDev *td);

	/** @brief set thermal trippoints
	 *
	 * @return integer with result of set thermal trippoints
	 * @retval NO_ACPI_ERROR
	 * @retval NO_MODULE_ERROR 
	 * @retval 3 if thermal device no. num does not exist 
	 */
	int setThermalTrippoints(const int zone, const ThermalDev td);

	/** @brief set cooling mode 
	*
	* @return integer value repesenting result of call func
	* @retval NO_ACPI_ERROR 
	* @retval NO_MODULE_ERROR
	* @retval 1 general error -> could not be set
	* @retval 2 if cooling_mode is not supported
	* @retval 3 if thermal device no. num does not exist
	*/
	int setCoolingMode(const int num, const int mode);
	/**************************************************
	 *                                               
	 *
	 * Functions: int getFanNum();
	 *	      int getFanStatus(const int no);
	 *
	 * Only ACPI!
	 * determines the fan's status and no. of devices
	 * 
	 *************************************************/

	/** @brief get number of fans in the system
	 *
	 * @return
	 *     - x < 0 on failure 
	 *     - the number of fan devices on success 
	 */
	int getFanNum(void);
	
	/** @brief get fan status
	 *
	 * @return integer with result of get fan status 
	 * @retval 0 if fan is on 
	 * @retval -1 if fan is off
	 * @retval 1 if no ACPI
	 * @retval 2 if no fan directory (module probably not loaded)
	 * @retval 3 if fan device no. num does not exist 
	 * @retval 4 if state file does not exist 
	 * @retval 5 if state could not be figured out
	 *           (probably changes in kernel syntax) 
	 */
	int getFanStatus(const int no);
	/******   O N L Y   F O R   A C P I ! ! !  *****************/


	/** @brief returns if the machine is on ac power or not
	 *
	 * (enum AC_ADAPTER_STATE{AC_ONLINE, AC_OFFLINE});
	 * apm + acpi
	 */
	int getACAdapterStatus(void);
	
	/******************************************************************/
	/* CPU FREQUENCY SCALING*/
	/* */
	/* all relevant code is inside the daemon for now*/
	/* the reason is too different implementations for 2.4.x and 2.6.x*/
	/* and mainly to not let other applications use it, because*/
	/* cpufreq scaling should always be controlled by only one application!*/
	/* therefore only general methods are implemented here!*/

	/** @brief get processor speed
	 * 
	 * be careful this method waits a second ...<br>
	 * first commented out because it failed on ia64 machines!
	 */
	float getRealProcessorSpeed(void);	
	
	/* returns UNKNOWN if not supported, just reads out /proc file*/
	
	/* returns "Idleness" of the processor in percent*/
	/* since the last time this function has been called*/
	
	/* if you poll idleness of processor, ignore the result */
	/* of the first time you invoke this function, because*/
	/* it will be the total idleness of cpu since the machine*/
	/* has been powered on*/
	double getCPULoad(void);
	double getCPULoadConsiderNice(const int nice);
	double getCPULoadSMP(const int consider_nice, const int cpu);
	void freeCPULoadSMPCalls(void);
	int getCPUCount(void);
	/******************************************************************/

	/******************************************************************/
        /*
         *  Sleeping States
         *
	 *
	 */

#define ACPI_S1 1
#define ACPI_S2 2
#define ACPI_S3 4
#define ACPI_S3_BIOS 8
#define ACPI_S4 16
#define ACPI_S4_BIOS 32
#define APM_STANDBY 64
#define APM_SUSPEND 128
	/** @brief mask out return value to evaluate supported states
	 *
	 * returns 0 if no support<br>
	 * see powerlib.h for mask to evaluate supported states
	 */
	int64_t getSupportedSleepStates(void);

	/** @brief determine whether there is apm, acpi or nothing on your Computer
	 *
	 * @return integer values representing acpi status
	 * @retval NOT_SUPPORTED=-1 APM
	 * @retval APM
	 * @retval ACPI
	 */
	int checkACPI(void);

	/** @brief  set debug level. Values: 0-31
	 *
	 * if library is compiled without -D DEBUG variable this method has no effect.
	 * 
	 * levels:
	 *     - 0 = nothing
	 *     - 1 = error
	 *     - 2 = warning
	 *     - 4 = diag
	 *     - 8 = info
	 *     - 16= debug
	 * sum up the levels you want to see: e.g.: 7 -> error + warning + diag
	 */
	void setDebugLevel(const int dbg);

	/***************** DEBUG DEFINITIONS ********************/
	/* define this variable through Makefile later to set debug possiblities on or of for the library*/
	/* additionally you can set -DSYSLOG in Makefile(see later) to let all debug be forwarded to */
	/* to the syslog daemon*/

#ifdef DEBUG
#define DBG_ERR 1
#define DBG_WARN 2
#define DBG_DIAG 4
#define DBG_INFO 8
#define DBG_DEBUG 16

	/* what about multiple threads ?!? -> I think it should be save ...*/
	extern int DEBUG_LEVEL;

#ifdef HAVE_SYSLOG_H
        static int syslog_open = 0; 
#define pDebug(level, string, args...) \
do{\
        if (!syslog_open){ \
                openlog("[powersave]",0 , LOG_DAEMON); \
                syslog_open = 1; \
        } \
        if (DEBUG_LEVEL & DBG_ERR & level){ \
              syslog(LOG_ERR,     "ERROR (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_WARN & level) { \
              syslog(LOG_WARNING, "WARNING (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_DIAG & level) { \
              syslog(LOG_NOTICE,  "DIAG (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_INFO & level) {\
              syslog(LOG_INFO,    "Info (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_DEBUG & level) {\
              syslog(LOG_INFO,    "Debug (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        } \
}while(0);
#else
#define pDebug(level, string, args...) \
do{\
        if (DEBUG_LEVEL & DBG_ERR & level){ \
              fprintf(stderr, "[POWERSAVE] ERROR (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_WARN & level) { \
              fprintf(stderr, "[POWERSAVE] WARNING (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_DIAG & level) { \
              printf("[POWERSAVE] DIAG (%s:%d) "string, __FUNCTION__, __LINE__, ## args); \
        }else if (DEBUG_LEVEL & DBG_INFO & level) {\
              printf(" "string, ## args); \
        }else {\
              printf( ""string, ## args); \
        } \
}while(0);
/*              printf("[POWERSAVE] INFO in Function %s, line %d: "string, __FUNCTION__, __LINE__, ## args); \*/
#endif

#else
#define pDebug(level, string, args...) do {} while (0);
#endif
	/***************** DEBUG DEFINITIONS ********************/

#ifdef __cplusplus
}
#endif

#endif

/** @} */ // end of libpower_information group
