#ifndef POWERSAVE_DBUS_H
#define POWERSAVE_DBUS_H

#define DBUS_API_SUBJECT_TO_CHANGE 1

#include <dbus-1.0/dbus/dbus.h>

#ifdef __cplusplus
extern "C" {
#endif

#define PS_DBUS_SERVICE                "com.novell.powersave"
#define PS_DBUS_ACTION_INTERFACE       "com.novell.powersave.action"
#define PS_DBUS_REQUEST_INTERFACE      "com.novell.powersave.request"
#define PS_DBUS_MANAGER_INTERFACE      "com.novell.powersave.manager"
#define PS_DBUS_EVENTS_INTERFACE      "com.novell.powersave.Events"
#define PS_DBUS_PATH                   "/com/novell/powersave"

/** @defgroup client_capabilities Client capabilities: Capabilities
 * clients can have
 * @ingroup client_exchange Defines the different
 * capabilities clients can request from the daemon.  Capabilities can be
 * added up. These are simple integer values.  @{
 */
/** Forward notifications the daemon throuws */
#define CAPABILITY_NOTIFICATIONS 1
/** Forward screenlock requests */
#define CAPABILITY_SCREENLOCK 2
/** @} */ // end of client_modes group

/** @brief dbus error message ids. They can be mapped
 *         to the DBus_Error_Array string array below
*/

enum DBUS_ERROR {REPLY_SUCCESS,
		 REPLY_NO_CONNECTION,
		 REPLY_NO_RIGHTS,
		 REPLY_INVALID_PARAM,
		 REPLY_INVALID_METHOD,
		 REPLY_INVALID_INTERFACE,
		 REPLY_HW_NOT_SUPPORTED,
		 REPLY_DISABLED,
		 REPLY_ALREADY_SET,
		 REPLY_DBUS_ERROR,
		 REPLY_GENERAL_ERROR,
		 REPLY_INVALID_MESSAGE_TYPE,
		 REPLY_ERROR_MAX};

extern char *DBus_Error_Array[REPLY_ERROR_MAX];

	
/** @brief Enum for the interface to use */
enum POWERSAVE_MESSAGE_TYPE {MANAGER_MESSAGE, REQUEST_MESSAGE, ACTION_MESSAGE, EVENTS_MESSAGE};

/** @brief if sendMessageWithReply was successful (returned REPLY_SUCCESS)
 *         you may want to use this function to get the contents of the reply.
 *
 *  @param msg: The message where the contents should be read from
 *  @param s:   The char pointer that gets filled with the desired string
 *  @param no:  The Xth string that will be added to the message that 
 *              s should be filled with (starting from 0)
 *
 *  @return negative on failure, zero on success
 */

int dbusGetMessageString(DBusMessage *msg, char** s, int no);
int dbusGetMessageInteger(DBusMessage *msg, int *i, int no);
int dbusGetMessageErrorID(DBusMessage *msg, unsigned *error_id);

/** @brief Send a message to the powersave daemon and receive the reply
 *         You can only use the REQUEST and ACTION interface (msg_type)
 *         and you need to know what request/action you want to send and give the
 *	   appropriate params for.
 *         Use the dbus type definitions and let the varargs end with
 *         DBUS_TYPE_INVALID (also see the dbus doc/code for the function
 *         dbus_message_append_args(...)
 *	   E.g.:
 *         @code
 *         DBusMessage *reply;
 *         int ret;
 *         char *scheme = "performance";
 *         ret = dbus_send_message(REQUEST, &reply, "set_scheme",
 *                                 DBUS_TYPE_STRING, &scheme, DBUS_TYPE_INVALID);
 *         if (ret == REPLY_SUCCESS){
 *                // PROCESS MESSAGE;
 *                dbus_message_unref(reply)
 *         }
 *         else{ // Handle error - no need to unref reply 
 *         } 
 *         @endcode
 *         Tries to set the current scheme ("set_scheme" -> method name) to
 *         "performance" (special argument for "set_scheme" messages)
 *
 *         @return The error code (see enum DBus_ERROR below. Use DBus_Error_Array[RET_VAL]
 *                 to get the corresponding error string).
 */
/* int dbus_send_message(POWERSAVE_MESSAGE_TYPE msg_type, int first_arg_type, ...); */

unsigned dbusSendSimpleMessage(int msg_type,
			  const char *method);

unsigned dbusSendSimpleMessageWithReply(int msg_type,
				   DBusMessage **reply,
				   const char *method);

unsigned dbusSendMessage(int msg_type,
			 const char *method,
			 int first_arg_type, ...);
	
unsigned dbusSendMessageWithReply(int msg_type,
				  DBusMessage **reply,
				  const char *method,
				  int first_arg_type, ...);


/** @brief establishes a connection to the daemon 
 *
 * @param capabilities sum of integers from powersave_clientsocket.h
 * @param connection the dbus connection from which the client's name is
 *        taken
 *
 * @return DBUS_ERROR
 */
dbus_int32_t establishConnection( int capabilities, DBusConnection *connection );	

	/* only for internal usage for above functions
unsigned dbusSendMessageGeneric(int msg_type,
				DBusMessage **reply,
				const char *method,
				va_list var_args);
	*/


#ifdef __cplusplus
}
#endif

#endif /*  POWERSAVE_DBUS_H */
